@@ -18,7 +18,7 @@ with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 import numpy as np
 import pandas as pd
-from .common import (
+from pybroker.common import (
     BarData,
     DataCol,
     ModelSymbol,
@@ -27,9 +27,10 @@ from .common import (
     to_datetime,
     to_decimal,
 )
-from .model import TrainedModel
-from .portfolio import Entry, Order, Portfolio, Position, Stop, Trade
-from .scope import (
+from pybroker.config import StrategyConfig
+from pybroker.model import TrainedModel
+from pybroker.portfolio import Entry, Order, Portfolio, Position, Stop, Trade
+from pybroker.scope import (
     ColumnScope,
     IndicatorScope,
     ModelInputScope,
@@ -56,10 +57,15 @@ from typing import (
 
 
 class BaseContext:
-    """Base context class."""
+    """Base context class.
+
+    Attributes:
+        config: :class:`pybroker.config.StrategyConfig`.
+    """
 
     def __init__(
         self,
+        config: StrategyConfig,
         portfolio: Portfolio,
         col_scope: ColumnScope,
         ind_scope: IndicatorScope,
@@ -69,6 +75,7 @@ class BaseContext:
         models: Mapping[ModelSymbol, TrainedModel],
         sym_end_index: Mapping[str, int],
     ):
+        self.config = config
         self._portfolio = portfolio
         self._col_scope = col_scope
         self._ind_scope = ind_scope
@@ -236,22 +243,26 @@ class BaseContext:
         if pos_type != "short" and pos_type != "long":
             raise ValueError(f"Unknown pos_type: {pos_type!r}.")
 
-    def calc_target_shares(self, target_size: float, price: float) -> int:
-        r"""Calculates the number of shares given a ``target_size`` equity
-        allocation and share ``price``.
+    def calc_target_shares(
+        self, target_size: float, price: float, cash: Optional[float] = None
+    ) -> int:
+        r"""Calculates the number of shares given a ``target_size`` allocation
+        and share ``price``.
 
         Args:
-            target_size: Amount of :class:`pybroker.portfolio.Portfolio` equity
-                used to calculate the number of shares, where the max
-                ``target_size`` is ``1``. For example, a ``target_size`` of
-                ``0.1`` would represent 10% of equity.
+            target_size: Proportion of cash used to calculate the number of
+                shares, where the max ``target_size`` is ``1``. For example, a
+                ``target_size`` of ``0.1`` would represent 10% of cash.
             price: Share price used to calculate the number of shares.
+            cash: Cash used to calculate the number of number of shares. If
+                ``None``, then the :class:`pybroker.portfolio.Portfolio` equity
+                is used to calculate the number of shares.
 
         Returns:
             Number of shares given ``target_size`` and share ``price``.
         """
         return int(
-            self._portfolio.equity
+            (to_decimal(cash) if cash is not None else self._portfolio.equity)
             * to_decimal(target_size)
             / to_decimal(price)
         )
@@ -345,6 +356,9 @@ class ExecResult:
             ``symbol``.
         long_stops: Stops for long :class:`pybroker.portfolio.Entry`\ s.
         short_stops: Stops for short :class:`pybroker.portfolio.Entry`\ s.
+        cover: Whether ``buy_shares`` are used to cover a short position. If
+            ``True``, the resulting buy order will be placed before sell
+            orders.
         pending_order_id: ID of :class:`pybroker.scope.PendingOrder` that was
             created.
     """
@@ -375,6 +389,7 @@ class ExecResult:
     sell_limit_price: Optional[Decimal]
     long_stops: Optional[frozenset[Stop]]
     short_stops: Optional[frozenset[Stop]]
+    cover: bool = field(default=False)
     pending_order_id: Optional[int] = field(default=None)
 
 
@@ -411,6 +426,7 @@ class PosSizeContext(BaseContext):
 
     def __init__(
         self,
+        config: StrategyConfig,
         portfolio: Portfolio,
         col_scope: ColumnScope,
         ind_scope: IndicatorScope,
@@ -420,10 +436,9 @@ class PosSizeContext(BaseContext):
         models: Mapping[ModelSymbol, TrainedModel],
         sessions: Mapping[str, Mapping],
         sym_end_index: Mapping[str, int],
-        max_long_positions: Optional[int],
-        max_short_positions: Optional[int],
     ):
         super().__init__(
+            config=config,
             portfolio=portfolio,
             col_scope=col_scope,
             ind_scope=ind_scope,
@@ -437,8 +452,8 @@ class PosSizeContext(BaseContext):
         self._signal_shares: dict[int, Union[int, float, Decimal]] = {}
         self._buy_results: Optional[list[ExecResult]] = None
         self._sell_results: Optional[list[ExecResult]] = None
-        self._max_long_positions = max_long_positions
-        self._max_short_positions = max_short_positions
+        self._max_long_positions = config.max_long_positions
+        self._max_short_positions = config.max_short_positions
 
     def signals(
         self, signal_type: Optional[Literal["buy", "sell"]] = None
@@ -583,6 +598,7 @@ class ExecContext(BaseContext):
     def __init__(
         self,
         symbol: str,
+        config: StrategyConfig,
         portfolio: Portfolio,
         col_scope: ColumnScope,
         ind_scope: IndicatorScope,
@@ -594,6 +610,7 @@ class ExecContext(BaseContext):
         session: Mapping,
     ):
         super().__init__(
+            config=config,
             portfolio=portfolio,
             col_scope=col_scope,
             ind_scope=ind_scope,
@@ -609,24 +626,28 @@ class ExecContext(BaseContext):
         self._foreign: dict[str, pd.DataFrame] = {}
 
         self.symbol: str = symbol
-        self.buy_fill_price: Union[
-            int,
-            float,
-            np.floating,
-            Decimal,
-            PriceType,
-            Callable[[str, BarData], Union[int, float, Decimal]],
-        ] = PriceType.MIDDLE
+        self.buy_fill_price: Optional[
+            Union[
+                int,
+                float,
+                np.floating,
+                Decimal,
+                PriceType,
+                Callable[[str, BarData], Union[int, float, Decimal]],
+            ]
+        ] = None
         self.buy_shares: Optional[Union[int, float, Decimal]] = None
         self.buy_limit_price: Optional[Union[int, float, Decimal]] = None
-        self.sell_fill_price: Union[
-            int,
-            float,
-            np.floating,
-            Decimal,
-            PriceType,
-            Callable[[str, BarData], Union[int, float, Decimal]],
-        ] = PriceType.MIDDLE
+        self.sell_fill_price: Optional[
+            Union[
+                int,
+                float,
+                np.floating,
+                Decimal,
+                PriceType,
+                Callable[[str, BarData], Union[int, float, Decimal]],
+            ]
+        ] = None
         self.sell_shares: Optional[Union[int, float, Decimal]] = None
         self.sell_limit_price: Optional[Union[int, float, Decimal]] = None
         self.hold_bars: Optional[int] = None
@@ -643,6 +664,8 @@ class ExecContext(BaseContext):
         self.stop_trailing_pct: Optional[Union[int, float, Decimal]] = None
         self.stop_trailing_limit: Optional[Union[int, float, Decimal]] = None
 
+        self._cover: bool = False
+
     def _verify_symbol(self):
         if self.symbol is None:
             raise ValueError("symbol is not set.")
@@ -731,6 +754,67 @@ class ExecContext(BaseContext):
             self._sym_end_index[self.symbol],
         )
 
+    @property
+    def cover_fill_price(
+        self,
+    ) -> Optional[
+        Union[
+            int,
+            float,
+            np.floating,
+            Decimal,
+            PriceType,
+            Callable[[str, BarData], Union[int, float, Decimal]],
+        ]
+    ]:
+        """Alias for :attr:`.buy_fill_price`. When set, this causes the buy
+        order to be placed before any sell orders.
+        """
+        return self.buy_fill_price
+
+    @cover_fill_price.setter
+    def cover_fill_price(
+        self,
+        fill_price: Optional[
+            Union[
+                int,
+                float,
+                np.floating,
+                Decimal,
+                PriceType,
+                Callable[[str, BarData], Union[int, float, Decimal]],
+            ]
+        ],
+    ):
+        self.buy_fill_price = fill_price
+        self._cover = True
+
+    @property
+    def cover_shares(self) -> Optional[Union[int, float, Decimal]]:
+        """Alias for :attr:`.buy_shares`. When set, this causes the buy
+        order to be placed before any sell orders.
+        """
+        return self.buy_shares
+
+    @cover_shares.setter
+    def cover_shares(self, shares: Optional[Union[int, float, Decimal]]):
+        self.buy_shares = shares
+        self._cover = True
+
+    @property
+    def cover_limit_price(self) -> Optional[Union[int, float, Decimal]]:
+        """Alias for :attr:`.buy_limit_price`. When set, this causes the buy
+        order to be placed before any sell orders.
+        """
+        return self.buy_limit_price
+
+    @cover_limit_price.setter
+    def cover_limit_price(
+        self, limit_price: Optional[Union[int, float, Decimal]]
+    ):
+        self.buy_limit_price = limit_price
+        self._cover = True
+
     def sell_all_shares(self):
         """Sells all long shares of :attr:`.ExecContext.symbol`."""
         pos = self.long_pos()
@@ -743,7 +827,7 @@ class ExecContext(BaseContext):
         pos = self.short_pos()
         if pos is None:
             return
-        self.buy_shares = pos.shares
+        self.cover_shares = pos.shares
 
     def foreign(
         self, symbol: str, col: Optional[str] = None
@@ -880,24 +964,30 @@ class ExecContext(BaseContext):
         return super().pos(symbol, "short")
 
     def calc_target_shares(
-        self, target_size: float, price: Optional[float] = None
+        self,
+        target_size: float,
+        price: Optional[float] = None,
+        cash: Optional[float] = None,
     ) -> int:
-        r"""Calculates the number of shares given a ``target_size`` equity
-        allocation and share ``price``.
+        r"""Calculates the number of shares given a ``target_size`` allocation
+        and share ``price``.
 
         Args:
-            target_size: Amount of :class:`pybroker.portfolio.Portfolio` equity
-                used to calculate the number of shares, where the max
-                ``target_size`` is ``1``. For example, a ``target_size`` of
-                ``0.1`` would result in 10% of equity.
+            target_size: Proportion of cash used to calculate the number of
+                shares, where the max ``target_size`` is ``1``. For example, a
+                ``target_size`` of ``0.1`` would represent 10% of cash.
             price: Share price used to calculate the number of shares. If
                 ``None``, the share price of the ``ExecContext``\ 's
                 :attr:`.symbol` is used.
+            cash: Cash used to calculate the number of number of shares. If
+                ``None``, then the :class:`pybroker.portfolio.Portfolio` equity
+                is used to calculate the number of shares.
+
         Returns:
             Number of shares given ``target_size`` and share ``price``.
         """
         price = self.close[-1] if price is None else price
-        return super().calc_target_shares(target_size, price)
+        return super().calc_target_shares(target_size, price, cash)
 
     def cancel_pending_order(self, order_id: int) -> bool:
         """Cancels a :class:`pybroker.scope.PendingOrder` with ``order_id``."""
@@ -1100,12 +1190,39 @@ class ExecContext(BaseContext):
                     limit_price=self.stop_trailing_limit,
                 )
             )
+        if (
+            self.stop_loss_limit is not None
+            and self.stop_loss is None
+            and self.stop_loss_pct is None
+        ):
+            raise ValueError(
+                "Either stop_loss or stop_loss_pct must be set when "
+                "stop_loss_limit is set."
+            )
+        if (
+            self.stop_profit_limit is not None
+            and self.stop_profit is None
+            and self.stop_profit_pct is None
+        ):
+            raise ValueError(
+                "Either stop_profit or stop_profit_pct must be set when "
+                "stop_profit_limit is set."
+            )
+        if (
+            self.stop_trailing_limit is not None
+            and self.stop_trailing is None
+            and self.stop_trailing_pct is None
+        ):
+            raise ValueError(
+                "Either stop_trailing or stop_trailing_pct must be set when "
+                "stop_trailing_limit is set."
+            )
         if pos_type == "long":
             return frozenset(stops), None
         else:
             return None, frozenset(stops)
 
-    def to_result(self) -> ExecResult:
+    def to_result(self) -> Optional[ExecResult]:
         """Creates an :class:`.ExecResult` from the data set on
         :class:`.ExecContext`.
         """
@@ -1113,6 +1230,64 @@ class ExecContext(BaseContext):
             raise ValueError("curr_date is not set.")
         if self.symbol is None:
             raise ValueError("symbol is not set.")
+        if self.buy_shares is None:
+            if self.buy_limit_price is not None:
+                raise ValueError(
+                    "buy_shares must be set when buy_limit_price is set."
+                )
+            if self.buy_fill_price is not None and self.hold_bars is None:
+                raise ValueError(
+                    "buy_shares or hold_bars must be set when "
+                    "buy_fill_price is set."
+                )
+        if self.sell_shares is None:
+            if self.sell_limit_price is not None:
+                raise ValueError(
+                    "sell_shares must be set when sell_limit_price is set."
+                )
+            if self.sell_fill_price is not None and self.hold_bars is None:
+                raise ValueError(
+                    "sell_shares or hold_bars must be set when "
+                    "sell_fill_price is set."
+                )
+        if self.buy_shares is None and self.sell_shares is None:
+            if (
+                self.stop_loss is not None
+                or self.stop_loss_pct is not None
+                or self.stop_loss_limit is not None
+                or self.stop_profit is not None
+                or self.stop_profit_pct is not None
+                or self.stop_profit_limit is not None
+                or self.stop_trailing is not None
+                or self.stop_trailing_pct is not None
+                or self.stop_trailing_limit is not None
+            ):
+                raise ValueError(
+                    "Either buy_shares or sell_shares must be set when a stop "
+                    "is set."
+                )
+            if self.hold_bars is not None:
+                raise ValueError(
+                    "Either buy_shares or sell_shares must be set when "
+                    "hold_bars is set."
+                )
+        if self.buy_shares is not None and self.sell_shares is not None:
+            raise ValueError(
+                "For each symbol, only one of buy_shares or sell_shares can be"
+                " set per bar."
+            )
+        if not self.buy_shares and not self.sell_shares:
+            return None
+        buy_fill_price = (
+            self.buy_fill_price
+            if self.buy_fill_price is not None
+            else PriceType.MIDDLE
+        )
+        sell_fill_price = (
+            self.sell_fill_price
+            if self.sell_fill_price is not None
+            else PriceType.MIDDLE
+        )
         buy_shares = (
             to_decimal(self.buy_shares)
             if self.buy_shares is not None
@@ -1137,8 +1312,8 @@ class ExecContext(BaseContext):
         return ExecResult(
             symbol=self.symbol,
             date=self._curr_date,
-            buy_fill_price=self.buy_fill_price,
-            sell_fill_price=self.sell_fill_price,
+            buy_fill_price=buy_fill_price,
+            sell_fill_price=sell_fill_price,
             score=self.score,
             hold_bars=self.hold_bars,
             buy_shares=buy_shares,
@@ -1147,6 +1322,7 @@ class ExecContext(BaseContext):
             sell_limit_price=sell_limit_price,
             long_stops=long_stops,
             short_stops=short_stops,
+            cover=self._cover,
         )
 
     def __getattr__(self, attr):
@@ -1169,10 +1345,11 @@ def set_exec_ctx_data(ctx: ExecContext, date: np.datetime64):
     ctx._curr_date = date
     ctx._dt = None
     ctx._foreign.clear()
-    ctx.buy_fill_price = PriceType.MIDDLE
+    ctx._cover = False
+    ctx.buy_fill_price = None
     ctx.buy_shares = None
     ctx.buy_limit_price = None
-    ctx.sell_fill_price = PriceType.MIDDLE
+    ctx.sell_fill_price = None
     ctx.sell_shares = None
     ctx.sell_limit_price = None
     ctx.hold_bars = None
