@@ -15,14 +15,15 @@ You should have received a copy of the GNU Lesser General Public License along
 with this program.  If not, see <https://www.gnu.org/licenses/>.
 """
 
+import akshare
 import alpaca.data.historical.stock as alpaca_stock
 import alpaca.data.historical.crypto as alpaca_crypto
 import itertools
 import numpy as np
 import pandas as pd
 import yfinance
-from .cache import DataSourceCacheKey
-from .common import (
+from pybroker.cache import DataSourceCacheKey
+from pybroker.common import (
     DataCol,
     parse_timeframe,
     to_datetime,
@@ -30,8 +31,9 @@ from .common import (
     verify_data_source_columns,
     verify_date_range,
 )
-from .scope import StaticScope
+from pybroker.scope import StaticScope
 from abc import ABC, abstractmethod
+from alpaca.data.enums import Adjustment
 from alpaca.data.requests import CryptoBarsRequest, StockBarsRequest
 from alpaca.data.timeframe import TimeFrame, TimeFrameUnit
 from datetime import datetime
@@ -49,6 +51,7 @@ class DataSourceCacheMixin:
         timeframe: str,
         start_date: Union[str, datetime, pd.Timestamp, np.datetime64],
         end_date: Union[str, datetime, pd.Timestamp, np.datetime64],
+        adjust: Optional[str],
     ) -> tuple[pd.DataFrame, Iterable[str]]:
         """Retrieves cached data from disk when caching is enabled with
         :meth:`pybroker.cache.enable_data_source_cache`.
@@ -69,6 +72,7 @@ class DataSourceCacheMixin:
                 An example timeframe string is ``1h 30m``.
             start_date: Starting date of the cached data (inclusive).
             end_date: Ending date of the cached data (inclusive).
+            adjust: The type of adjustment to make.
 
         Returns:
             ``tuple[pandas.DataFrame, Iterable[str]]`` containing a
@@ -92,6 +96,7 @@ class DataSourceCacheMixin:
                 tf_seconds=tf_seconds,
                 start_date=start_date,
                 end_date=end_date,
+                adjust=adjust,
             )
             cached = cache.get(repr(cache_key))
             scope.logger.debug_get_data_source_cache(cache_key)
@@ -115,6 +120,7 @@ class DataSourceCacheMixin:
         timeframe: str,
         start_date: Union[str, datetime, pd.Timestamp, np.datetime64],
         end_date: Union[str, datetime, pd.Timestamp, np.datetime64],
+        adjust: Optional[str],
         data: pd.DataFrame,
     ):
         """Stores data to disk cache when caching is enabled with
@@ -135,6 +141,7 @@ class DataSourceCacheMixin:
                 An example timeframe string would be ``1h 30m``.
             start_date: Starting date of the data to cache (inclusive).
             end_date: Ending date of the data to cache (inclusive).
+            adjust: The type of adjustment to make.
             data: :class:`pandas.DataFrame` containing the data to cache.
         """
         if data.empty:
@@ -153,6 +160,7 @@ class DataSourceCacheMixin:
                 tf_seconds=tf_seconds,
                 start_date=start_date,
                 end_date=end_date,
+                adjust=adjust,
             )
             cache.set(repr(cache_key), df)
             scope.logger.debug_set_data_source_cache(cache_key)
@@ -175,6 +183,7 @@ class DataSource(ABC, DataSourceCacheMixin):
         start_date: Union[str, datetime],
         end_date: Union[str, datetime],
         timeframe: Optional[str] = "",
+        adjust: Optional[str] = None,
     ) -> pd.DataFrame:
         """Queries data. Cached data is returned if caching is enabled by
         calling :meth:`pybroker.cache.enable_data_source_cache`.
@@ -195,6 +204,7 @@ class DataSource(ABC, DataSourceCacheMixin):
                 - ``"mo"``/``"month"``: months
 
                 An example timeframe string is ``1h 30m``.
+            adjust: The type of adjustment to make.
 
         Returns:
             :class:`pandas.DataFrame` containing the queried data.
@@ -217,6 +227,7 @@ class DataSource(ABC, DataSourceCacheMixin):
             timeframe=timeframe,
             start_date=start_date,
             end_date=end_date,
+            adjust=adjust,
         )
         if not uncached_syms:
             return cached_df
@@ -227,7 +238,9 @@ class DataSource(ABC, DataSourceCacheMixin):
             start_date=start_date,
             end_date=end_date,
         )
-        df = self._fetch_data(unique_syms, start_date, end_date, timeframe)
+        df = self._fetch_data(
+            unique_syms, start_date, end_date, timeframe, adjust
+        )
         if (
             self._scope.data_source_cache is not None
             and not cached_df.columns.empty
@@ -237,7 +250,7 @@ class DataSource(ABC, DataSourceCacheMixin):
             self._scope.data_source_cache.clear()
             return self.query(symbols, start_date, end_date, timeframe)
         verify_data_source_columns(df)
-        self.set_cached(timeframe, start_date, end_date, df)
+        self.set_cached(timeframe, start_date, end_date, adjust, df)
         df = pd.concat((cached_df, df))
         if not df.empty:
             df = df.sort_values(by=[DataCol.DATE.value, DataCol.SYMBOL.value])
@@ -251,6 +264,7 @@ class DataSource(ABC, DataSourceCacheMixin):
         start_date: datetime,
         end_date: datetime,
         timeframe: Optional[str],
+        adjust: Optional[str],
     ) -> pd.DataFrame:
         """:meta public:
         Override this method to return data from a custom
@@ -274,6 +288,7 @@ class DataSource(ABC, DataSourceCacheMixin):
                 - ``"mo"``/``"month"``: months
 
                 An example timeframe string is ``1h 30m``.
+            adjust: The type of adjustment to make.
 
         Returns:
             :class:`pandas.DataFrame` containing the queried data.
@@ -326,9 +341,10 @@ class Alpaca(DataSource):
         start_date: Union[str, datetime],
         end_date: Union[str, datetime],
         timeframe: Optional[str] = "1d",
+        adjust: Optional[str] = None,
     ) -> pd.DataFrame:
         _parse_alpaca_timeframe(timeframe)
-        return super().query(symbols, start_date, end_date, timeframe)
+        return super().query(symbols, start_date, end_date, timeframe, adjust)
 
     def _fetch_data(
         self,
@@ -336,16 +352,25 @@ class Alpaca(DataSource):
         start_date: datetime,
         end_date: datetime,
         timeframe: Optional[str],
+        adjust: Optional[str],
     ) -> pd.DataFrame:
         """:meta private:"""
         amount, unit = _parse_alpaca_timeframe(timeframe)
+        adj_enum = None
+        if adjust is not None:
+            for member in Adjustment:
+                if member.value == adjust:
+                    adj_enum = member
+                    break
+            if adj_enum is None:
+                raise ValueError(f"Unknown adjustment: {adjust}.")
         request = StockBarsRequest(
             symbol_or_symbols=list(symbols),
             start=start_date,
             end=end_date,
             timeframe=TimeFrame(amount, unit),
             limit=None,
-            adjustment=None,
+            adjustment=adj_enum,
             feed=None,
         )
         df = self._api.get_stock_bars(request).df  # type: ignore[union-attr]
@@ -410,9 +435,10 @@ class AlpacaCrypto(DataSource):
         start_date: Union[str, datetime],
         end_date: Union[str, datetime],
         timeframe: Optional[str] = "1d",
+        _: Optional[str] = None,
     ) -> pd.DataFrame:
         _parse_alpaca_timeframe(timeframe)
-        return super().query(symbols, start_date, end_date, timeframe)
+        return super().query(symbols, start_date, end_date, timeframe, _)
 
     def _fetch_data(
         self,
@@ -420,6 +446,7 @@ class AlpacaCrypto(DataSource):
         start_date: datetime,
         end_date: datetime,
         timeframe: Optional[str],
+        _: Optional[str],
     ) -> pd.DataFrame:
         """:meta private:"""
         amount, unit = _parse_alpaca_timeframe(timeframe)
@@ -464,7 +491,8 @@ class YFinance(DataSource):
         symbols: Union[str, Iterable[str]],
         start_date: Union[str, datetime],
         end_date: Union[str, datetime],
-        _: Optional[str] = "",
+        _timeframe: Optional[str] = "",
+        _adjust: Optional[str] = None,
     ) -> pd.DataFrame:
         r"""Queries data from `Yahoo Finance <https://finance.yahoo.com/>`_\ .
         The timeframe of the data is limited to per day only.
@@ -477,14 +505,17 @@ class YFinance(DataSource):
         Returns:
             :class:`pandas.DataFrame` containing the queried data.
         """
-        return super().query(symbols, start_date, end_date, self.__TIMEFRAME)
+        return super().query(
+            symbols, start_date, end_date, self.__TIMEFRAME, _adjust
+        )
 
     def _fetch_data(
         self,
         symbols: frozenset[str],
         start_date: datetime,
         end_date: datetime,
-        _: Optional[str],
+        _timeframe: Optional[str],
+        _adjust: Optional[str],
     ) -> pd.DataFrame:
         """:meta private:"""
         df = yfinance.download(list(symbols), start=start_date, end=end_date)
@@ -532,3 +563,104 @@ class YFinance(DataSource):
                 sym_df[self.ADJ_CLOSE] = df[("Adj Close", sym)].values
                 result = pd.concat((result, sym_df))
         return result
+
+
+class AKShare(DataSource):
+    r"""Retrieves data from `AKShare <https://akshare.akfamily.xyz/>`_\ .
+
+    Attributes:
+        ADJ_CLOSE: Column name of adjusted close prices.
+    """
+
+    __TIMEFRAME: Final = "1d"
+
+    def query(
+        self,
+        symbols: Union[str, Iterable[str]],
+        start_date: Union[str, datetime],
+        end_date: Union[str, datetime],
+        _: Optional[str] = "",
+        adjust: Optional[str] = "qfq",
+    ) -> pd.DataFrame:
+        r"""Queries data from `AKShare <https://akshare.akfamily.xyz/>`_\ .
+        The timeframe of the data is limited to per day only.
+
+        Args:
+            symbols: Ticker symbols of the data to query.
+            start_date: Start date of the data to query (inclusive).
+            end_date: End date of the data to query (inclusive).
+            adjust: The type of adjustment to make.
+
+        Returns:
+            :class:`pandas.DataFrame` containing the queried data.
+        """
+        return super().query(
+            symbols, start_date, end_date, self.__TIMEFRAME, adjust
+        )
+
+    def _fetch_data(
+        self,
+        symbols: frozenset[str],
+        start_date: datetime,
+        end_date: datetime,
+        _: Optional[str],
+        adjust: Optional[str],
+    ) -> pd.DataFrame:
+        """:meta private:"""
+        start_date_str = to_datetime(start_date).strftime("%Y%m%d")
+        end_date_str = to_datetime(end_date).strftime("%Y%m%d")
+        symbols_list = list(symbols)
+        symbols_simple = [item.split(".")[0] for item in symbols_list]
+        result = pd.DataFrame()
+        for i in range(len(symbols_list)):
+            try:
+                temp_df = akshare.stock_zh_a_hist(
+                    symbols_simple[i],
+                    start_date=start_date_str,
+                    end_date=end_date_str,
+                    period="daily",
+                    adjust=adjust if adjust is not None else "",
+                )
+                if not temp_df.columns.empty:
+                    temp_df["symbol"] = symbols_list[i]
+            except KeyError:
+                temp_df = pd.DataFrame()
+            result = pd.concat([result, temp_df], ignore_index=True)
+        if result.columns.empty:
+            return pd.DataFrame(
+                columns=[
+                    DataCol.SYMBOL.value,
+                    DataCol.DATE.value,
+                    DataCol.OPEN.value,
+                    DataCol.HIGH.value,
+                    DataCol.LOW.value,
+                    DataCol.CLOSE.value,
+                    DataCol.VOLUME.value,
+                ]
+            )
+        if result.empty:
+            return result
+        result.rename(
+            columns={
+                "日期": DataCol.DATE.value,
+                "开盘": DataCol.OPEN.value,
+                "收盘": DataCol.CLOSE.value,
+                "最高": DataCol.HIGH.value,
+                "最低": DataCol.LOW.value,
+                "成交量": DataCol.VOLUME.value,
+            },
+            inplace=True,
+        )
+        result["date"] = pd.to_datetime(result["date"])
+        result = result[
+            [
+                DataCol.DATE.value,
+                DataCol.SYMBOL.value,
+                DataCol.OPEN.value,
+                DataCol.HIGH.value,
+                DataCol.LOW.value,
+                DataCol.CLOSE.value,
+                DataCol.VOLUME.value,
+            ]
+        ]
+        return result
