@@ -20,8 +20,15 @@ with this program.  If not, see <https://www.gnu.org/licenses/>.
 import itertools
 import numpy as np
 import pandas as pd
-from .common import BarData, DataCol, FeeMode, PriceType, StopType, to_decimal
-from .scope import PriceScope, StaticScope
+from pybroker.common import (
+    BarData,
+    DataCol,
+    FeeMode,
+    PriceType,
+    StopType,
+    to_decimal,
+)
+from pybroker.scope import PriceScope, StaticScope
 from collections import deque
 from dataclasses import dataclass, field
 from decimal import Decimal
@@ -906,13 +913,13 @@ class Portfolio:
                 if close is not None:
                     pos.close = close
                     pos.pnl = _calculate_pnl(close, pos.entries, "short")
-                    pos.market_value = pos.pnl
                     pos.margin = close * pos.shares
+                    pos.market_value = pos.margin + pos.pnl
                     pos_short_shares += pos.shares
-                    pos_market_value += pos.pnl
+                    pos_market_value += pos.market_value
                     pos_pnl += pos.pnl
                     pos_margin += pos.margin
-                total_market_value += pos.market_value
+                total_market_value -= pos.margin
                 total_margin += pos.margin
             if close is not None:
                 self.position_bars.append(
