@@ -18,8 +18,8 @@ with this program.  If not, see <https://www.gnu.org/licenses/>.
 import dataclasses
 import numpy as np
 import pandas as pd
-from .cache import CacheDateFields
-from .common import (
+from pybroker.cache import CacheDateFields
+from pybroker.common import (
     BarData,
     DataCol,
     Day,
@@ -33,20 +33,26 @@ from .common import (
     verify_data_source_columns,
     verify_date_range,
 )
-from .config import StrategyConfig
-from .context import (
+from pybroker.config import StrategyConfig
+from pybroker.context import (
     ExecContext,
     ExecResult,
     PosSizeContext,
     set_exec_ctx_data,
     set_pos_size_ctx_data,
 )
-from .data import AlpacaCrypto, DataSource
-from .eval import BootstrapResult, EvalMetrics, EvaluateMixin
-from .indicator import Indicator, IndicatorsMixin
-from .model import ModelSource, ModelsMixin, TrainedModel
-from .portfolio import Order, Portfolio, PortfolioBar, PositionBar, Trade
-from .scope import (
+from pybroker.data import AlpacaCrypto, DataSource
+from pybroker.eval import BootstrapResult, EvalMetrics, EvaluateMixin
+from pybroker.indicator import Indicator, IndicatorsMixin
+from pybroker.model import ModelSource, ModelsMixin, TrainedModel
+from pybroker.portfolio import (
+    Order,
+    Portfolio,
+    PortfolioBar,
+    PositionBar,
+    Trade,
+)
+from pybroker.scope import (
     ColumnScope,
     IndicatorScope,
     ModelInputScope,
@@ -55,6 +61,7 @@ from .scope import (
     PriceScope,
     StaticScope,
 )
+from pybroker.slippage import SlippageData, SlippageModel
 from collections import defaultdict, deque
 from dataclasses import dataclass
 from datetime import datetime
@@ -113,6 +120,7 @@ class BacktestMixin:
 
     def backtest_executions(
         self,
+        config: StrategyConfig,
         executions: set[Execution],
         before_exec_fn: Optional[Callable[[Mapping[str, ExecContext]], None]],
         after_exec_fn: Optional[Callable[[Mapping[str, ExecContext]], None]],
@@ -121,24 +129,17 @@ class BacktestMixin:
         indicator_data: Mapping[IndicatorSymbol, pd.Series],
         test_data: pd.DataFrame,
         portfolio: Portfolio,
-        buy_delay: int,
-        sell_delay: int,
-        max_long_positions: Optional[int],
-        max_short_positions: Optional[int],
         pos_size_handler: Optional[Callable[[PosSizeContext], None]],
         exit_dates: Mapping[str, np.datetime64],
-        exit_buy_fill_price: Union[
-            PriceType, Callable[[str, BarData], Union[int, float, Decimal]]
-        ] = PriceType.MIDDLE,
-        exit_sell_fill_price: Union[
-            PriceType, Callable[[str, BarData], Union[int, float, Decimal]]
-        ] = PriceType.MIDDLE,
+        slippage_model: Optional[SlippageModel] = None,
         enable_fractional_shares: bool = False,
+        warmup: Optional[int] = None,
     ):
         r"""Backtests a ``set`` of :class:`.Execution`\ s that implement
         trading logic.
 
         Args:
+            config: :class:`pybroker.config.StrategyConfig`.
             executions: :class:`.Execution`\ s to run.
             sessions: :class:`Mapping` of symbols to :class:`Mapping` of custom
                 data that persists for every bar during the
@@ -151,22 +152,13 @@ class BacktestMixin:
                 values.
             test_data: :class:`pandas.DataFrame` of test data.
             portfolio: :class:`pybroker.portfolio.Portfolio`.
-            buy_delay: Number of bars before placing an order for a buy signal.
-            sell_delay: Number of bars before placing an order for a sell
-                signal.
-            max_long_positions: Maximum number of long positions that can be
-                held at a time. If ``None``, then unlimited.
-            max_short_positions: Maximum number of short positions that can be
-                held at a time. If ``None``, then unlimited.
             pos_size_handler: :class:`Callable` that sets position sizes when
                 placing orders for buy and sell signals.
             exit_dates: :class:`Mapping` of symbols to exit dates.
-            exit_buy_fill_price: Fill price for covering an open short position
-                for ``exit_dates``.
-            exit_sell_fill_price: Fill price for selling an open long position
-                for ``exit_dates``.
             enable_fractional_shares: Whether to enable trading fractional
                 shares.
+            warmup: Number of bars that need to pass before running the
+                executions.
 
         Returns:
             :class:`.TestResult` of the backtest.
@@ -183,7 +175,7 @@ class BacktestMixin:
         col_scope = ColumnScope(test_data)
         price_scope = PriceScope(col_scope, sym_end_index)
         ind_scope = IndicatorScope(indicator_data, test_dates)
-        input_scope = ModelInputScope(col_scope, ind_scope)
+        input_scope = ModelInputScope(col_scope, ind_scope, models)
         pred_scope = PredictionScope(models, input_scope)
         pending_order_scope = PendingOrderScope()
         exec_ctxs: dict[str, ExecContext] = {}
@@ -194,6 +186,7 @@ class BacktestMixin:
                     continue
                 exec_ctxs[sym] = ExecContext(
                     symbol=sym,
+                    config=config,
                     portfolio=portfolio,
                     col_scope=col_scope,
                     ind_scope=ind_scope,
@@ -210,10 +203,12 @@ class BacktestMixin:
             sym: frozenset(test_data.loc[pd.IndexSlice[sym, :]].index.values)
             for sym in exec_ctxs.keys()
         }
+        cover_sched: dict[np.datetime64, list[ExecResult]] = defaultdict(list)
         buy_sched: dict[np.datetime64, list[ExecResult]] = defaultdict(list)
         sell_sched: dict[np.datetime64, list[ExecResult]] = defaultdict(list)
         if pos_size_handler is not None:
             pos_ctx = PosSizeContext(
+                config=config,
                 portfolio=portfolio,
                 col_scope=col_scope,
                 ind_scope=ind_scope,
@@ -223,48 +218,69 @@ class BacktestMixin:
                 models=models,
                 sessions=sessions,
                 sym_end_index=sym_end_index,
-                max_long_positions=max_long_positions,
-                max_short_positions=max_short_positions,
             )
         logger = StaticScope.instance().logger
         logger.backtest_executions_start(test_dates)
+        cover_results: deque[ExecResult] = deque()
         buy_results: deque[ExecResult] = deque()
         sell_results: deque[ExecResult] = deque()
-        exit_syms: deque[str] = deque()
+        exit_ctxs: deque[ExecContext] = deque()
         active_ctxs: dict[str, ExecContext] = {}
         for i, date in enumerate(test_dates):
             active_ctxs.clear()
             for sym, ctx in exec_ctxs.items():
                 if date not in sym_exec_dates[sym]:
                     continue
-                active_ctxs[sym] = ctx
                 sym_end_index[sym] += 1
+                if warmup and sym_end_index[sym] <= warmup:
+                    continue
+                active_ctxs[sym] = ctx
                 set_exec_ctx_data(ctx, date)
                 if (
                     exit_dates
                     and sym in exit_dates
                     and date == exit_dates[sym]
                 ):
-                    exit_syms.append(sym)
+                    exit_ctxs.append(ctx)
+            is_cover_sched = date in cover_sched
             is_buy_sched = date in buy_sched
             is_sell_sched = date in sell_sched
-            if is_buy_sched and (
-                max_long_positions is not None or pos_size_handler is not None
+            if (
+                config.max_long_positions is not None
+                or pos_size_handler is not None
             ):
-                buy_sched[date].sort(key=_sort_by_score, reverse=True)
+                if is_cover_sched:
+                    cover_sched[date].sort(key=_sort_by_score, reverse=True)
+                elif is_buy_sched:
+                    buy_sched[date].sort(key=_sort_by_score, reverse=True)
             if is_sell_sched and (
-                max_short_positions is not None or pos_size_handler is not None
+                config.max_short_positions is not None
+                or pos_size_handler is not None
             ):
                 sell_sched[date].sort(key=_sort_by_score, reverse=True)
             if pos_size_handler is not None and (
-                is_buy_sched or is_sell_sched
+                is_cover_sched or is_buy_sched or is_sell_sched
             ):
+                pos_size_buy_results = None
+                if is_cover_sched:
+                    pos_size_buy_results = cover_sched[date]
+                elif is_buy_sched:
+                    pos_size_buy_results = buy_sched[date]
                 self._set_pos_sizes(
                     pos_size_handler=pos_size_handler,
                     pos_ctx=pos_ctx,
-                    buy_results=buy_sched[date] if is_buy_sched else None,
+                    buy_results=pos_size_buy_results,
                     sell_results=sell_sched[date] if is_sell_sched else None,
                 )
+            if is_cover_sched:
+                self._place_buy_orders(
+                    date=date,
+                    price_scope=price_scope,
+                    pending_order_scope=pending_order_scope,
+                    buy_sched=cover_sched,
+                    portfolio=portfolio,
+                    enable_fractional_shares=enable_fractional_shares,
+                )
             if is_sell_sched:
                 self._place_sell_orders(
                     date=date,
@@ -285,27 +301,42 @@ class BacktestMixin:
                 )
             portfolio.check_stops(date, price_scope)
             portfolio.capture_bar(date, test_data)
-            if before_exec_fn is not None:
+            if before_exec_fn is not None and active_ctxs:
                 before_exec_fn(active_ctxs)
             for sym, ctx in active_ctxs.items():
                 if sym in exec_fns:
                     exec_fns[sym](ctx)
-            if after_exec_fn is not None:
+            if after_exec_fn is not None and active_ctxs:
                 after_exec_fn(active_ctxs)
             for ctx in active_ctxs.values():
-                result = self._to_result(ctx)
+                if slippage_model and (ctx.buy_shares or ctx.sell_shares):
+                    self._apply_slippage(slippage_model, ctx)
+                result = ctx.to_result()
                 if result is None:
                     continue
                 if result.buy_shares is not None:
-                    buy_results.append(result)
+                    if result.cover:
+                        cover_results.append(result)
+                    else:
+                        buy_results.append(result)
                 if result.sell_shares is not None:
                     sell_results.append(result)
+            while cover_results:
+                self._schedule_order(
+                    result=cover_results.popleft(),
+                    created=date,
+                    sym_end_index=sym_end_index,
+                    delay=config.buy_delay,
+                    sched=cover_sched,
+                    col_scope=col_scope,
+                    pending_order_scope=pending_order_scope,
+                )
             while buy_results:
                 self._schedule_order(
                     result=buy_results.popleft(),
                     created=date,
                     sym_end_index=sym_end_index,
-                    delay=buy_delay,
+                    delay=config.buy_delay,
                     sched=buy_sched,
                     col_scope=col_scope,
                     pending_order_scope=pending_order_scope,
@@ -315,48 +346,40 @@ class BacktestMixin:
                     result=sell_results.popleft(),
                     created=date,
                     sym_end_index=sym_end_index,
-                    delay=sell_delay,
+                    delay=config.sell_delay,
                     sched=sell_sched,
                     col_scope=col_scope,
                     pending_order_scope=pending_order_scope,
                 )
-            while exit_syms:
+            while exit_ctxs:
                 self._exit_position(
                     portfolio=portfolio,
                     date=date,
-                    symbol=exit_syms.popleft(),
-                    exit_buy_fill_price=exit_buy_fill_price,
-                    exit_sell_fill_price=exit_sell_fill_price,
+                    ctx=exit_ctxs.popleft(),
+                    exit_cover_fill_price=config.exit_cover_fill_price,
+                    exit_sell_fill_price=config.exit_sell_fill_price,
                     price_scope=price_scope,
                 )
             portfolio.incr_bars()
             if i % 10 == 0 or i == len(test_dates) - 1:
                 logger.backtest_executions_loading(i + 1)
 
-    def _to_result(self, ctx: ExecContext) -> Optional[ExecResult]:
-        if ctx.buy_shares is not None and ctx.sell_shares is not None:
-            raise ValueError(
-                "For each symbol, only one of buy_shares or sell_shares can be"
-                " set per bar."
-            )
-        if ctx.buy_limit_price is not None and ctx.buy_shares is None:
-            raise ValueError(
-                "buy_shares must be set when buy_limit_price is set."
-            )
-        if ctx.sell_limit_price is not None and ctx.sell_shares is None:
-            raise ValueError(
-                "sell_shares must be set when sell_limit_price is set."
-            )
-        if not ctx.buy_shares and not ctx.sell_shares:
-            return None
-        return ctx.to_result()
+    def _apply_slippage(
+        self,
+        slippage_model: SlippageModel,
+        ctx: ExecContext,
+    ):
+        buy_shares = to_decimal(ctx.buy_shares) if ctx.buy_shares else None
+        sell_shares = to_decimal(ctx.sell_shares) if ctx.sell_shares else None
+        data = SlippageData(buy_shares=buy_shares, sell_shares=sell_shares)
+        slippage_model.apply_slippage(data, ctx)
 
     def _exit_position(
         self,
         portfolio: Portfolio,
         date: np.datetime64,
-        symbol: str,
-        exit_buy_fill_price: Union[
+        ctx: ExecContext,
+        exit_cover_fill_price: Union[
             PriceType, Callable[[str, BarData], Union[int, float, Decimal]]
         ],
         exit_sell_fill_price: Union[
@@ -364,11 +387,11 @@ class BacktestMixin:
         ],
         price_scope: PriceScope,
     ):
-        buy_fill_price = price_scope.fetch(symbol, exit_buy_fill_price)
-        sell_fill_price = price_scope.fetch(symbol, exit_sell_fill_price)
+        buy_fill_price = price_scope.fetch(ctx.symbol, exit_cover_fill_price)
+        sell_fill_price = price_scope.fetch(ctx.symbol, exit_sell_fill_price)
         portfolio.exit_position(
             date,
-            symbol,
+            ctx.symbol,
             buy_fill_price=buy_fill_price,
             sell_fill_price=sell_fill_price,
         )
@@ -805,6 +828,7 @@ class Strategy(
         self._pos_size_handler: Optional[
             Callable[[PosSizeContext], None]
         ] = None
+        self._slippage_model: Optional[SlippageModel] = None
         self._scope = StaticScope.instance()
         self._logger = self._scope.logger
 
@@ -838,6 +862,10 @@ class Strategy(
         elif not isinstance(data_source, DataSource):
             raise TypeError(f"Invalid data_source type: {type(data_source)}")
 
+    def set_slippage_model(self, slippage_model: Optional[SlippageModel]):
+        """Sets :class:`pybroker.slippage.SlippageModel`."""
+        self._slippage_model = slippage_model
+
     def add_execution(
         self,
         fn: Optional[Callable[[ExecContext], None]],
@@ -980,8 +1008,9 @@ class Strategy(
         lookahead: int = 1,
         train_size: int = 0,
         shuffle: bool = False,
-        calc_bootstrap: bool = True,
+        calc_bootstrap: bool = False,
         disable_parallel: bool = False,
+        warmup: Optional[int] = None,
     ) -> Optional[TestResult]:
         """Backtests the trading strategy by running executions that were added
         with :meth:`.add_execution`.
@@ -1023,12 +1052,14 @@ class Strategy(
                 Defaults to ``False``. Disabled when model caching is enabled
                 via :meth:`pybroker.cache.enable_model_cache`.
             calc_bootstrap: Whether to compute randomized bootstrap evaluation
-                metrics. Defaults to ``True``.
+                metrics. Defaults to ``False``.
             disable_parallel: If ``True``,
                 :class:`pybroker.indicator.Indicator` data is computed
                 serially. If ``False``, :class:`pybroker.indicator.Indicator`
                 data is computed in parallel using multiple processes.
                 Defaults to ``False``.
+            warmup: Number of bars that need to pass before running the
+                executions.
 
         Returns:
             :class:`.BacktestResult` containing portfolio balances, order
@@ -1046,6 +1077,7 @@ class Strategy(
             shuffle=shuffle,
             calc_bootstrap=calc_bootstrap,
             disable_parallel=disable_parallel,
+            warmup=warmup,
         )
 
     def walkforward(
@@ -1059,8 +1091,9 @@ class Strategy(
         days: Optional[Union[str, Day, Iterable[Union[str, Day]]]] = None,
         train_size: float = 0.5,
         shuffle: bool = False,
-        calc_bootstrap: bool = True,
+        calc_bootstrap: bool = False,
         disable_parallel: bool = False,
+        warmup: Optional[int] = None,
     ) -> Optional[TestResult]:
         """Backtests the trading strategy using `Walkforward Analysis
         <https://www.pybroker.com/en/latest/notebooks/6.%20Training%20a%20Model.html#Walkforward-Analysis>`_.
@@ -1108,17 +1141,21 @@ class Strategy(
                 Defaults to ``False``. Disabled when model caching is enabled
                 via :meth:`pybroker.cache.enable_model_cache`.
             calc_bootstrap: Whether to compute randomized bootstrap evaluation
-                metrics. Defaults to ``True``.
+                metrics. Defaults to ``False``.
             disable_parallel: If ``True``,
                 :class:`pybroker.indicator.Indicator` data is computed
                 serially. If ``False``, :class:`pybroker.indicator.Indicator`
                 data is computed in parallel using multiple processes.
                 Defaults to ``False``.
+            warmup: Number of bars that need to pass before running the
+                executions.
 
         Returns:
             :class:`.BacktestResult` containing portfolio balances, order
             history, and evaluation metrics.
         """
+        if warmup is not None and warmup < 1:
+            raise ValueError("warmup must be > 0.")
         scope = StaticScope.instance()
         try:
             scope.freeze_data_cols()
@@ -1191,6 +1228,7 @@ class Strategy(
                 train_size=train_size,
                 shuffle=shuffle,
                 train_only=train_only,
+                warmup=warmup,
             )
             if train_only:
                 self._logger.walkforward_completed()
@@ -1234,6 +1272,7 @@ class Strategy(
         train_size: float,
         shuffle: bool,
         train_only: bool,
+        warmup: Optional[int],
     ):
         sessions: dict[str, dict] = defaultdict(dict)
         exit_dates: dict[str, np.datetime64] = {}
@@ -1281,6 +1320,7 @@ class Strategy(
                 )
             if not train_only and not test_data.empty:
                 self.backtest_executions(
+                    config=self._config,
                     executions=self._executions,
                     before_exec_fn=self._before_exec_fn,
                     after_exec_fn=self._after_exec_fn,
@@ -1289,15 +1329,11 @@ class Strategy(
                     indicator_data=indicator_data,
                     test_data=test_data,
                     portfolio=portfolio,
-                    buy_delay=self._config.buy_delay,
-                    sell_delay=self._config.sell_delay,
-                    max_long_positions=self._config.max_long_positions,
-                    max_short_positions=self._config.max_short_positions,
                     pos_size_handler=self._pos_size_handler,
                     exit_dates=exit_dates,
+                    slippage_model=self._slippage_model,
                     enable_fractional_shares=self._fractional_shares_enabled(),
-                    exit_buy_fill_price=self._config.exit_cover_fill_price,
-                    exit_sell_fill_price=self._config.exit_sell_fill_price,
+                    warmup=warmup,
                 )
 
     def _filter_dates(
