@@ -15,6 +15,7 @@ You should have received a copy of the GNU Lesser General Public License along
 with this program.  If not, see <https://www.gnu.org/licenses/>.
 """
 
+import akshare
 import joblib
 import os
 import pandas as pd
@@ -25,7 +26,13 @@ from .fixtures import *  # noqa: F401
 from datetime import datetime
 from pybroker.cache import DataSourceCacheKey
 from pybroker.common import to_seconds
-from pybroker.data import Alpaca, AlpacaCrypto, DataSourceCacheMixin, YFinance
+from pybroker.data import (
+    AKShare,
+    Alpaca,
+    AlpacaCrypto,
+    DataSourceCacheMixin,
+    YFinance,
+)
 from unittest import mock
 
 API_KEY = "api_key"
@@ -34,6 +41,7 @@ API_VERSION = "v2"
 TIMEFRAME = "1m"
 START_DATE = datetime.strptime("2021-02-02", "%Y-%m-%d")
 END_DATE = datetime.strptime("2022-02-02", "%Y-%m-%d")
+ADJUST = "all"
 ALPACA_COLS = [
     "date",
     "symbol",
@@ -124,7 +132,9 @@ class TestDataSourceCacheMixin:
     @pytest.mark.usefixtures("scope")
     def test_set_cached(self, alpaca_df, symbols, mock_cache):
         cache_mixin = DataSourceCacheMixin()
-        cache_mixin.set_cached(TIMEFRAME, START_DATE, END_DATE, alpaca_df)
+        cache_mixin.set_cached(
+            TIMEFRAME, START_DATE, END_DATE, ADJUST, alpaca_df
+        )
         assert len(mock_cache.set.call_args_list) == len(symbols)
         for i, sym in enumerate(symbols):
             expected_cache_key = DataSourceCacheKey(
@@ -132,6 +142,7 @@ class TestDataSourceCacheMixin:
                 tf_seconds=to_seconds(TIMEFRAME),
                 start_date=START_DATE,
                 end_date=END_DATE,
+                adjust=ADJUST,
             )
             cache_key, sym_df = mock_cache.set.call_args_list[i].args
             assert cache_key == repr(expected_cache_key)
@@ -144,7 +155,7 @@ class TestDataSourceCacheMixin:
     def test_get_cached_when_empty(self, mock_cache, query_symbols):
         cache_mixin = DataSourceCacheMixin()
         df, uncached_syms = cache_mixin.get_cached(
-            query_symbols, TIMEFRAME, START_DATE, END_DATE
+            query_symbols, TIMEFRAME, START_DATE, END_DATE, ADJUST
         )
         assert df.empty
         assert uncached_syms == query_symbols
@@ -155,6 +166,7 @@ class TestDataSourceCacheMixin:
                 tf_seconds=to_seconds(TIMEFRAME),
                 start_date=START_DATE,
                 end_date=END_DATE,
+                adjust=ADJUST,
             )
             cache_key = mock_cache.get.call_args_list[i].args[0]
             assert cache_key == repr(expected_cache_key)
@@ -162,9 +174,11 @@ class TestDataSourceCacheMixin:
     @pytest.mark.usefixtures("setup_enabled_ds_cache")
     def test_set_and_get_cached(self, alpaca_df, symbols):
         cache_mixin = DataSourceCacheMixin()
-        cache_mixin.set_cached(TIMEFRAME, START_DATE, END_DATE, alpaca_df)
+        cache_mixin.set_cached(
+            TIMEFRAME, START_DATE, END_DATE, ADJUST, alpaca_df
+        )
         df, uncached_syms = cache_mixin.get_cached(
-            symbols, TIMEFRAME, START_DATE, END_DATE
+            symbols, TIMEFRAME, START_DATE, END_DATE, ADJUST
         )
         assert df.equals(alpaca_df)
         assert not len(uncached_syms)
@@ -173,9 +187,11 @@ class TestDataSourceCacheMixin:
     def test_set_and_get_cached_when_partial(self, alpaca_df, symbols):
         cache_mixin = DataSourceCacheMixin()
         cached_df = alpaca_df[alpaca_df["symbol"].isin(symbols[:2])]
-        cache_mixin.set_cached(TIMEFRAME, START_DATE, END_DATE, cached_df)
+        cache_mixin.set_cached(
+            TIMEFRAME, START_DATE, END_DATE, ADJUST, cached_df
+        )
         df, uncached_syms = cache_mixin.get_cached(
-            symbols, TIMEFRAME, START_DATE, END_DATE
+            symbols, TIMEFRAME, START_DATE, END_DATE, ADJUST
         )
         assert df.equals(cached_df)
         assert uncached_syms == symbols[2:]
@@ -209,15 +225,21 @@ class TestDataSourceCacheMixin:
     ):
         cache_mixin = DataSourceCacheMixin()
         with pytest.raises(error):
-            cache_mixin.set_cached(timeframe, start_date, end_date, alpaca_df)
+            cache_mixin.set_cached(
+                timeframe, start_date, end_date, ADJUST, alpaca_df
+            )
         with pytest.raises(error):
-            cache_mixin.get_cached(symbols, timeframe, start_date, end_date)
+            cache_mixin.get_cached(
+                symbols, timeframe, start_date, end_date, ADJUST
+            )
 
     def test_set_and_get_cached_when_cache_disabled(self, alpaca_df, symbols):
         cache_mixin = DataSourceCacheMixin()
-        cache_mixin.set_cached(TIMEFRAME, START_DATE, END_DATE, alpaca_df)
+        cache_mixin.set_cached(
+            TIMEFRAME, START_DATE, END_DATE, ADJUST, alpaca_df
+        )
         df, uncached_syms = cache_mixin.get_cached(
-            symbols, TIMEFRAME, START_DATE, END_DATE
+            symbols, TIMEFRAME, START_DATE, END_DATE, ADJUST
         )
         assert df.empty
         assert uncached_syms == symbols
@@ -236,7 +258,9 @@ class TestAlpaca:
         with mock.patch.object(
             alpaca._api, "get_stock_bars", return_value=mock_bars
         ):
-            df = alpaca.query(symbols, START_DATE, END_DATE, TIMEFRAME)
+            df = alpaca.query(
+                symbols, START_DATE, END_DATE, TIMEFRAME, adjust="all"
+            )
             df = (
                 df.sort_values(["symbol", "date"])
                 .reset_index(drop=True)
@@ -249,19 +273,29 @@ class TestAlpaca:
             )
             assert df.equals(expected)
 
+    def test_query_when_invalid_adj_then_error(self, symbols):
+        alpaca = Alpaca(API_KEY, API_SECRET)
+        with pytest.raises(
+            ValueError,
+            match=re.escape("Unknown adjustment: foo"),
+        ):
+            alpaca.query(
+                symbols, START_DATE, END_DATE, TIMEFRAME, adjust="foo"
+            )
+
     @pytest.mark.usefixtures(
         "setup_enabled_ds_cache", "mock_alpaca", "tmp_path"
     )
     def test_query_when_partial_cache(self, alpaca_df, bars_df, symbols):
         alpaca = Alpaca(API_KEY, API_SECRET)
         cached_df = alpaca_df[alpaca_df["symbol"].isin(symbols[-1:])]
-        alpaca.set_cached(TIMEFRAME, START_DATE, END_DATE, cached_df)
+        alpaca.set_cached(TIMEFRAME, START_DATE, END_DATE, ADJUST, cached_df)
         mock_bars = mock.Mock()
         mock_bars.df = bars_df[bars_df["symbol"].isin(symbols[:-1])]
         with mock.patch.object(
             alpaca._api, "get_stock_bars", return_value=mock_bars
         ):
-            df = alpaca.query(symbols, START_DATE, END_DATE, TIMEFRAME)
+            df = alpaca.query(symbols, START_DATE, END_DATE, TIMEFRAME, ADJUST)
             df = (
                 df.sort_values(["symbol", "date"])
                 .reset_index(drop=True)
@@ -281,13 +315,13 @@ class TestAlpaca:
         alpaca = Alpaca(API_KEY, API_SECRET)
         cached_df = alpaca_df[alpaca_df["symbol"].isin(symbols[-1:])]
         cached_df = cached_df.drop(columns=["vwap"])
-        alpaca.set_cached(TIMEFRAME, START_DATE, END_DATE, cached_df)
+        alpaca.set_cached(TIMEFRAME, START_DATE, END_DATE, ADJUST, cached_df)
         mock_bars = mock.Mock()
         mock_bars.df = bars_df[bars_df["symbol"].isin(symbols[:-1])]
         with mock.patch.object(
             alpaca._api, "get_stock_bars", return_value=mock_bars
         ):
-            df = alpaca.query(symbols, START_DATE, END_DATE, TIMEFRAME)
+            df = alpaca.query(symbols, START_DATE, END_DATE, TIMEFRAME, ADJUST)
             assert not df.empty
             assert set(df.columns) == set(
                 (
@@ -542,3 +576,74 @@ class TestYFinance:
                 "adj_close",
             )
         )
+
+
+class TestAKShare:
+    @pytest.mark.usefixtures("setup_ds_cache")
+    def test_query(self):
+        symbols = ["A"]
+        ak = AKShare()
+        expected_df = pd.DataFrame(
+            {
+                "日期": [END_DATE],
+                "开盘": [1],
+                "收盘": [2],
+                "最高": [3],
+                "最低": [4],
+                "成交量": [5],
+                "symbol": symbols,
+            }
+        )
+        with mock.patch.object(
+            akshare, "stock_zh_a_hist", return_value=expected_df
+        ):
+            df = ak.query(symbols, START_DATE, END_DATE)
+        assert set(df.columns) == {
+            "date",
+            "open",
+            "high",
+            "low",
+            "close",
+            "volume",
+            "symbol",
+        }
+        assert df.shape[0] == expected_df.shape[0]
+        assert set(df["symbol"].unique()) == set(symbols)
+        assert (df["date"].unique() == expected_df["日期"].unique()).all()
+
+    @pytest.mark.parametrize(
+        "columns",
+        [
+            [],
+            [
+                "date",
+                "open",
+                "high",
+                "low",
+                "close",
+                "volume",
+                "symbol",
+            ],
+        ],
+    )
+    @pytest.mark.usefixtures("setup_ds_cache")
+    def test_query_when_empty_result(self, columns):
+        ak = AKShare()
+        with mock.patch.object(
+            akshare,
+            "stock_zh_a_hist",
+            return_value=pd.DataFrame(columns=columns),
+        ):
+            df = ak.query(["A"], START_DATE, END_DATE)
+        assert df.empty
+        assert set(df.columns) == set(
+            (
+                "date",
+                "open",
+                "high",
+                "low",
+                "close",
+                "volume",
+                "symbol",
+            )
+        )
