@@ -177,7 +177,7 @@ def test_sharpe_ratio(values, expected_sharpe):
 @pytest.mark.parametrize(
     "values, expected_dd",
     [
-        ([0, 0.1, 0.15, -0.05, 0.1, -0.25, -0.15, 0], -0.4),
+        ([0.1, 0.15, -0.05, 0.1, -0.25, -0.15, 0], -0.4),
         ([0.1, -0.4], -0.4),
         ([1, 1, 1, 1], 0),
         ([1], 0),
