@@ -17,6 +17,7 @@ with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 import numpy as np
 import pandas as pd
+import pybroker
 import pytest
 import re
 from .fixtures import *
@@ -30,6 +31,7 @@ from pybroker.scope import (
     enable_progress_bar,
     disable_logging,
     disable_progress_bar,
+    param,
     register_columns,
     unregister_columns,
 )
@@ -108,6 +110,23 @@ def test_disable_progress_bar(mock_logger):
     mock_logger.disable_progress_bar.assert_called_once()
 
 
+def test_param_when_empty():
+    assert param("bar") is None
+
+
+@pytest.mark.parametrize("value", [42, None])
+def test_param_when_set_and_get(value):
+    param("foo", value)
+    assert param("foo") == value
+
+
+def test_param_when_set_to_none():
+    param("baz", 11)
+    assert param("baz") == 11
+    param("baz", None)
+    assert param("baz") is None
+
+
 class TestStaticScope:
     def test_set_and_get_indicator(self, scope, hhv_ind):
         scope.set_indicator(hhv_ind)
@@ -261,30 +280,23 @@ class TestModelInputScope:
         self,
         scope,
         indicators,
-        ind_names,
         input_scope,
         symbol,
         data_source_df,
         end_index,
+        trained_model,
     ):
         scope.custom_data_cols = set()
-        expected_cols = set(ind_names) | {
-            "date",
-            "open",
-            "high",
-            "low",
-            "close",
-            "volume",
-        }
+        expected_cols = {"hhv", "llv", "sumv"}
 
         def input_fn(df):
             assert set(df.columns) == expected_cols
-            df["foo"] = np.ones(len(df["close"]))
+            df["foo"] = np.ones(len(df["hhv"]))
             return df
 
         model_source = model(
-            "model",
-            lambda symbol, *_: {MODEL_NAME: symbol},
+            trained_model.name,
+            lambda *_: trained_model,
             indicators,
             input_data_fn=input_fn,
         )
@@ -374,7 +386,9 @@ class TestPredictionScope:
             pred_scope.fetch(sym, name)
 
     def test_fetch_when_predict_not_defined_then_error(self, input_scope):
-        model = TrainedModel(name=MODEL_NAME, instance={}, predict_fn=None)
+        model = TrainedModel(
+            name=MODEL_NAME, instance={}, predict_fn=None, input_cols=None
+        )
         pred_scope = PredictionScope(
             models={ModelSymbol(MODEL_NAME, "SPY"): model},
             input_scope=input_scope,
@@ -389,6 +403,26 @@ class TestPredictionScope:
         ):
             pred_scope.fetch("SPY", MODEL_NAME)
 
+    def test_fetch_when_input_data_empty_then_error(self, col_scope):
+        model_name = "no_input_data"
+        ind_scope = IndicatorScope({}, [])
+        pybroker.model(model_name, lambda sym, train, test: {})
+        model = TrainedModel(
+            name=model_name, instance={}, predict_fn=None, input_cols=None
+        )
+        models = {ModelSymbol(model_name, "SPY"): model}
+        input_scope = ModelInputScope(col_scope, ind_scope, models)
+        pred_scope = PredictionScope(models, input_scope)
+        with pytest.raises(
+            ValueError,
+            match=re.escape(
+                f"No input data found for model {model_name!r}. Consider "
+                "passing input_data_fn to pybroker#model() if custom columns "
+                "were registered."
+            ),
+        ):
+            pred_scope.fetch("SPY", model_name)
+
 
 class TestPriceScope:
     @pytest.mark.parametrize(
