@@ -37,6 +37,7 @@ from pybroker.portfolio import (
     Trade,
 )
 from pybroker.scope import PendingOrder
+from pybroker.slippage import SlippageData, SlippageModel
 from pybroker.strategy import (
     BacktestMixin,
     Execution,
@@ -198,6 +199,7 @@ class TestBacktestMixin:
         mock_portfolio = Mock()
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -206,10 +208,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=data_source_df,
             portfolio=mock_portfolio,
-            buy_delay=1,
-            sell_delay=1,
-            max_long_positions=None,
-            max_short_positions=None,
             pos_size_handler=pos_size_handler,
             exit_dates={},
         )
@@ -238,6 +236,78 @@ class TestBacktestMixin:
             )
             assert kwargs["limit_price"] == 50.5
 
+    def test_backtest_when_pos_size_handler_and_cover(self, data_source_df):
+        def pos_size_handler(ctx):
+            signals = tuple(ctx.signals())
+            ctx.set_shares(signals[0], shares=10)
+            ctx.set_shares(signals[1], shares=20)
+            assert type(ctx.sessions["SPY"]) == dict
+            assert type(ctx.sessions["AAPL"]) == dict
+
+        def buy_exec_fn(ctx):
+            ctx.cover_fill_price = 1
+            ctx.cover_shares = 2
+
+        def sell_exec_fn(ctx):
+            ctx.sell_fill_price = 1
+            ctx.sell_shares = 1
+
+        buy_exec = Execution(
+            id=1,
+            symbols=frozenset(["SPY"]),
+            fn=buy_exec_fn,
+            model_names=frozenset(),
+            indicator_names=frozenset(),
+        )
+        sell_exec = Execution(
+            id=2,
+            symbols=frozenset(["AAPL"]),
+            fn=sell_exec_fn,
+            model_names=frozenset(),
+            indicator_names=frozenset(),
+        )
+        execs = {buy_exec, sell_exec}
+        portfolio = Portfolio(1_000_000)
+        mixin = BacktestMixin()
+        mixin.backtest_executions(
+            config=StrategyConfig(),
+            executions=execs,
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
+            models={},
+            indicator_data={},
+            test_data=data_source_df,
+            portfolio=portfolio,
+            pos_size_handler=pos_size_handler,
+            exit_dates={},
+        )
+        buy_df = data_source_df[data_source_df["symbol"] == "SPY"]
+        buy_dates = buy_df["date"].unique()[1:]
+        sell_df = data_source_df[data_source_df["symbol"] == "AAPL"]
+        sell_dates = sell_df["date"].unique()[1:]
+        assert len(portfolio.orders) == len(buy_dates) + len(sell_dates)
+        assert len(
+            list(
+                filter(
+                    lambda o: o.type == "buy"
+                    and o.symbol == "SPY"
+                    and o.shares == 10,
+                    portfolio.orders,
+                )
+            )
+        ) == len(buy_dates)
+        assert len(
+            list(
+                filter(
+                    lambda o: o.type == "sell"
+                    and o.symbol == "AAPL"
+                    and o.shares == 20,
+                    portfolio.orders,
+                )
+            )
+        ) == len(sell_dates)
+
     def test_backtest_executions_when_buy_delay(self, data_source_df):
         def buy_exec_fn(ctx):
             ctx.buy_fill_price = PriceType.CLOSE
@@ -255,6 +325,7 @@ class TestBacktestMixin:
         mock_portfolio = Mock()
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(buy_delay=2),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -263,10 +334,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=data_source_df,
             portfolio=mock_portfolio,
-            buy_delay=2,
-            sell_delay=1,
-            max_long_positions=None,
-            max_short_positions=None,
             pos_size_handler=None,
             exit_dates={},
         )
@@ -300,6 +367,7 @@ class TestBacktestMixin:
         mock_portfolio = Mock()
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(sell_delay=2),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -308,10 +376,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=data_source_df,
             portfolio=mock_portfolio,
-            buy_delay=1,
-            sell_delay=2,
-            max_long_positions=None,
-            max_short_positions=None,
             pos_size_handler=None,
             exit_dates={},
         )
@@ -348,6 +412,7 @@ class TestBacktestMixin:
             ValueError, match=re.escape("hold_bars must be greater than 0.")
         ):
             mixin.backtest_executions(
+                config=StrategyConfig(),
                 executions=execs,
                 before_exec_fn=None,
                 after_exec_fn=None,
@@ -356,10 +421,6 @@ class TestBacktestMixin:
                 indicator_data={},
                 test_data=data_source_df,
                 portfolio=Mock(),
-                buy_delay=1,
-                sell_delay=1,
-                max_long_positions=None,
-                max_short_positions=None,
                 pos_size_handler=None,
                 exit_dates={},
             )
@@ -384,6 +445,7 @@ class TestBacktestMixin:
             ValueError, match=re.escape("hold_bars must be greater than 0.")
         ):
             mixin.backtest_executions(
+                config=StrategyConfig(),
                 executions=execs,
                 before_exec_fn=None,
                 after_exec_fn=None,
@@ -392,10 +454,6 @@ class TestBacktestMixin:
                 indicator_data={},
                 test_data=data_source_df,
                 portfolio=Mock(),
-                buy_delay=1,
-                sell_delay=1,
-                max_long_positions=None,
-                max_short_positions=None,
                 pos_size_handler=None,
                 exit_dates={},
             )
@@ -412,6 +470,7 @@ class TestBacktestMixin:
         portfolio = Portfolio(100_000)
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -420,10 +479,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=data_source_df,
             portfolio=portfolio,
-            buy_delay=1,
-            sell_delay=1,
-            max_long_positions=None,
-            max_short_positions=None,
             pos_size_handler=None,
             exit_dates={},
         )
@@ -447,6 +502,7 @@ class TestBacktestMixin:
         portfolio = Portfolio(100_000)
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -455,10 +511,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=data_source_df[data_source_df["symbol"] != "AAPL"],
             portfolio=portfolio,
-            buy_delay=1,
-            sell_delay=1,
-            max_long_positions=None,
-            max_short_positions=None,
             pos_size_handler=None,
             exit_dates={},
         )
@@ -484,6 +536,7 @@ class TestBacktestMixin:
         portfolio = Portfolio(100_000)
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(buy_delay=1_000),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -492,10 +545,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=data_source_df,
             portfolio=portfolio,
-            buy_delay=1000,
-            sell_delay=1,
-            max_long_positions=None,
-            max_short_positions=None,
             pos_size_handler=None,
             exit_dates={},
         )
@@ -522,6 +571,7 @@ class TestBacktestMixin:
         portfolio = Portfolio(100_000)
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(sell_delay=1000),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -530,10 +580,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=data_source_df,
             portfolio=portfolio,
-            buy_delay=1,
-            sell_delay=1000,
-            max_long_positions=None,
-            max_short_positions=None,
             pos_size_handler=None,
             exit_dates={},
         )
@@ -562,6 +608,7 @@ class TestBacktestMixin:
         mock_portfolio = Mock()
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(max_long_positions=1),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -570,10 +617,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=data_source_df,
             portfolio=mock_portfolio,
-            buy_delay=1,
-            sell_delay=1,
-            max_long_positions=1,
-            max_short_positions=None,
             pos_size_handler=None,
             exit_dates={},
         )
@@ -615,6 +658,7 @@ class TestBacktestMixin:
         mock_portfolio = Mock()
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(max_short_positions=1),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -623,10 +667,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=data_source_df,
             portfolio=mock_portfolio,
-            buy_delay=1,
-            sell_delay=1,
-            max_long_positions=None,
-            max_short_positions=1,
             pos_size_handler=None,
             exit_dates={},
         )
@@ -648,6 +688,106 @@ class TestBacktestMixin:
             )
             assert kwargs["limit_price"] is None
 
+    def test_backtest_executions_when_max_short_positions_and_cover(
+        self, data_source_df
+    ):
+        def sell_exec_fn(ctx):
+            if ctx.symbol == "AAPL":
+                if ctx.bars == 1:
+                    ctx.sell_shares = 200
+                elif ctx.bars == 2:
+                    ctx.cover_all_shares()
+            else:
+                if ctx.bars == 2:
+                    ctx.sell_shares = 100
+
+        exec = Execution(
+            id=1,
+            symbols=frozenset(["AAPL", "SPY"]),
+            fn=sell_exec_fn,
+            model_names=frozenset(),
+            indicator_names=frozenset(),
+        )
+        execs = {exec}
+        portfolio = Portfolio(100_000, max_short_positions=1)
+        mixin = BacktestMixin()
+        mixin.backtest_executions(
+            config=StrategyConfig(max_short_positions=1),
+            executions=execs,
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
+            models={},
+            indicator_data={},
+            test_data=data_source_df,
+            portfolio=portfolio,
+            pos_size_handler=None,
+            exit_dates={},
+        )
+        assert len(portfolio.short_positions) == 1
+        assert not portfolio.long_positions
+        assert len(portfolio.orders) == 3
+        orders = portfolio.orders
+        assert orders[0].symbol == "AAPL"
+        assert orders[0].shares == 200
+        assert orders[0].type == "sell"
+        assert orders[1].symbol == "AAPL"
+        assert orders[1].shares == 200
+        assert orders[1].type == "buy"
+        assert orders[2].symbol == "SPY"
+        assert orders[2].shares == 100
+        assert orders[2].type == "sell"
+        trades = portfolio.trades
+        assert len(trades) == 1
+        assert trades[0].symbol == "AAPL"
+        assert trades[0].type == "short"
+
+    def test_backtest_executions_when_max_long_positions_and_cover(
+        self, data_source_df
+    ):
+        def cover_exec_fn(ctx):
+            if ctx.symbol == "AAPL":
+                ctx.score = 2
+            else:
+                ctx.score = 1
+            ctx.cover_shares = 100
+            ctx.hold_bars = 1
+
+        exec = Execution(
+            id=1,
+            symbols=frozenset(["AAPL", "SPY"]),
+            fn=cover_exec_fn,
+            model_names=frozenset(),
+            indicator_names=frozenset(),
+        )
+        execs = {exec}
+        portfolio = Portfolio(100_000, max_long_positions=1)
+        mixin = BacktestMixin()
+        mixin.backtest_executions(
+            config=StrategyConfig(max_long_positions=1),
+            executions=execs,
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
+            models={},
+            indicator_data={},
+            test_data=data_source_df,
+            portfolio=portfolio,
+            pos_size_handler=None,
+            exit_dates={},
+        )
+        dates = data_source_df["date"].unique()[1:]
+        orders = portfolio.orders
+        assert (
+            len(list(filter(lambda o: o.symbol == "AAPL", orders)))
+            == len(dates) * 2 - 1
+        )
+        trades = portfolio.trades
+        assert (
+            len(list(filter(lambda t: t.symbol == "AAPL", trades)))
+            == len(dates) - 1
+        )
+
     @pytest.mark.parametrize(
         "price_type, expected_fill_price",
         [
@@ -693,6 +833,7 @@ class TestBacktestMixin:
         mock_portfolio = Mock()
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -701,10 +842,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=df,
             portfolio=mock_portfolio,
-            buy_delay=1,
-            sell_delay=1,
-            max_long_positions=None,
-            max_short_positions=None,
             pos_size_handler=None,
             exit_dates={},
         )
@@ -736,6 +873,7 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         with pytest.raises(ValueError, match=r"Unknown price: .*"):
             mixin.backtest_executions(
+                config=StrategyConfig(),
                 executions=execs,
                 before_exec_fn=None,
                 after_exec_fn=None,
@@ -744,10 +882,6 @@ class TestBacktestMixin:
                 indicator_data={},
                 test_data=data_source_df,
                 portfolio=Portfolio(100_000),
-                buy_delay=1,
-                sell_delay=1,
-                max_long_positions=None,
-                max_short_positions=None,
                 pos_size_handler=None,
                 exit_dates={},
             )
@@ -774,6 +908,7 @@ class TestBacktestMixin:
             ),
         ):
             mixin.backtest_executions(
+                config=StrategyConfig(),
                 executions=execs,
                 before_exec_fn=None,
                 after_exec_fn=None,
@@ -782,10 +917,6 @@ class TestBacktestMixin:
                 indicator_data={},
                 test_data=data_source_df,
                 portfolio=Portfolio(100_000),
-                buy_delay=1,
-                sell_delay=1,
-                max_long_positions=1,
-                max_short_positions=None,
                 pos_size_handler=None,
                 exit_dates={},
             )
@@ -812,6 +943,7 @@ class TestBacktestMixin:
             ),
         ):
             mixin.backtest_executions(
+                config=StrategyConfig(),
                 executions=execs,
                 before_exec_fn=None,
                 after_exec_fn=None,
@@ -820,10 +952,6 @@ class TestBacktestMixin:
                 indicator_data={},
                 test_data=data_source_df,
                 portfolio=Portfolio(100_000),
-                buy_delay=1,
-                sell_delay=1,
-                max_long_positions=1,
-                max_short_positions=None,
                 pos_size_handler=None,
                 exit_dates={},
             )
@@ -846,6 +974,7 @@ class TestBacktestMixin:
         portfolio = Portfolio(1)
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -854,10 +983,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=data_source_df,
             portfolio=portfolio,
-            buy_delay=1,
-            sell_delay=1,
-            max_long_positions=1,
-            max_short_positions=None,
             pos_size_handler=None,
             exit_dates={},
         )
@@ -882,6 +1007,7 @@ class TestBacktestMixin:
         portfolio = Portfolio(1)
         mixin = BacktestMixin()
         mixin.backtest_executions(
+            config=StrategyConfig(),
             executions=execs,
             before_exec_fn=None,
             after_exec_fn=None,
@@ -890,10 +1016,6 @@ class TestBacktestMixin:
             indicator_data={},
             test_data=data_source_df,
             portfolio=portfolio,
-            buy_delay=1,
-            sell_delay=1,
-            max_long_positions=1,
-            max_short_positions=None,
             pos_size_handler=None,
             exit_dates={},
         )
@@ -1012,7 +1134,9 @@ def between_time(request):
 
 
 class FakeDataSource(DataSource):
-    def _fetch_data(self, symbols, start_date, end_date, timeframe):
+    def _fetch_data(
+        self, symbols, start_date, end_date, timeframe, adjustment
+    ):
         return joblib.load(
             os.path.join(os.path.dirname(__file__), "testdata/daily_1.joblib")
         )
@@ -1209,7 +1333,7 @@ class TestStrategy:
         strategy = Strategy(data_source_df, START_DATE, END_DATE)
         for exec in executions_only:
             strategy.add_execution(**exec)
-        result = strategy.backtest()
+        result = strategy.backtest(calc_bootstrap=True)
         assert type(result) == TestResult
         assert result.start_date == datetime.strptime(START_DATE, "%Y-%m-%d")
         assert result.end_date == datetime.strptime(END_DATE, "%Y-%m-%d")
@@ -1814,6 +1938,52 @@ class TestStrategy:
         assert len(result.trades) == len(buy_orders)
         assert (result.trades["stop"] == "bar").all()
 
+    def test_backtest_when_slippage(self, data_source_df):
+        class FakeSlippageModel(SlippageModel):
+            def apply_slippage(self, data: SlippageData, ctx: ExecContext):
+                ctx.buy_shares = 99
+
+        def buy_exec_fn(ctx):
+            ctx.buy_fill_price = PriceType.CLOSE
+            ctx.sell_fill_price = PriceType.OPEN
+            ctx.buy_shares = 100
+            ctx.hold_bars = 2
+
+        df = data_source_df[data_source_df["symbol"] == "SPY"]
+        dates = df["date"].unique()
+        dates = dates[dates <= np.datetime64(END_DATE)]
+        buy_dates = dates[1:]
+        sell_dates = dates[3:]
+        config = StrategyConfig(initial_cash=500_000)
+        strategy = Strategy(data_source_df, START_DATE, END_DATE, config)
+        strategy.set_slippage_model(FakeSlippageModel())
+        strategy.add_execution(buy_exec_fn, "SPY")
+        result = strategy.backtest(calc_bootstrap=False)
+        orders = result.orders
+        buy_orders = orders[orders["type"] == "buy"]
+        assert len(buy_orders) == len(buy_dates)
+        for buy_date in buy_dates:
+            row = buy_orders[buy_orders["date"] == buy_date]
+            assert row["symbol"].item() == "SPY"
+            assert row["shares"].item() == 99
+            assert np.isnan(row["limit_price"].item())
+            assert row["fill_price"].item() == round(
+                df[df["date"] == buy_date]["close"].item(), 2
+            )
+            assert row["fees"].item() == 0
+        sell_orders = orders[orders["type"] == "sell"]
+        assert len(sell_orders) == len(sell_dates)
+        for sell_date in sell_dates:
+            row = sell_orders[sell_orders["date"] == sell_date]
+            assert row["symbol"].item() == "SPY"
+            assert row["shares"].item() == 99
+            assert np.isnan(row["limit_price"].item())
+            assert row["fill_price"].item() == round(
+                df[df["date"] == sell_date]["open"].item(), 2
+            )
+            assert row["fees"].item() == 0
+        assert (result.trades["stop"] == "bar").all()
+
     def test_backtest_when_stop_loss(self, data_source_df):
         def exec_fn(ctx):
             if ctx.bars == 1:
@@ -2100,3 +2270,24 @@ class TestStrategy:
         assert buy_order["shares"] == 200
         assert np.isnan(buy_order["limit_price"])
         assert buy_order["fees"] == 0
+
+    def test_backtest_when_warmup(self, data_source_df):
+        def exec_fn(ctx):
+            if ctx.bars <= 10:
+                raise AssertionError("Warmup failed.")
+            elif not ctx.long_pos():
+                ctx.buy_shares = 100
+
+        strategy = Strategy(data_source_df, START_DATE, END_DATE)
+        strategy.add_execution(exec_fn, "SPY")
+        result = strategy.backtest(warmup=10)
+        assert len(result.orders) == 1
+
+    def test_backtest_when_warmup_invalid_then_error(self, data_source_df):
+        def exec_fn(ctx):
+            pass
+
+        strategy = Strategy(data_source_df, START_DATE, END_DATE)
+        strategy.add_execution(exec_fn, "SPY")
+        with pytest.raises(ValueError, match=re.escape("warmup must be > 0.")):
+            strategy.backtest(warmup=-1)
